/* parser.mly */

%{ open Ast %}
%{ open Type %}

%token <string> ID
%token RETURN IF ELSE FOR WHILE BREAK VOID
%token INT FLOAT BOOL CHAR STRING CELL LINE TABLE
%token <int> INTLITERAL
%token <float> FLOATLITERAL
%token <char> CHARLITERAL
%token <string> STRINGLITERAL
%token <bool> BOOLLITERAL
%token NULL VOID
%token SEMI LBRACE RBRACE COMMA
%token ASSIGN
%token RANGE 
%token AND NOT OR REF
%token EQ NEQ LT LEQ GT GEQ
%token PLUS MINUS TIMES DIVIDE MOD
%token LSQPAREN RSQPAREN WILDCARD FUNC
%token PRINT 
%token LPAREN RPAREN
%token EOF

%nonassoc NOELSE
%nonassoc ELSE

%right ASSIGN 
%left OR
%left AND
%left EQ NEQ
%left LT GT LEQ GEQ 
%left PLUS MINUS
%left TIMES DIVIDE MOD
%right NOT REF
%left LPAREN

%start program
%type <Ast.program> program

%%

program:
	program_r		{ List.rev $1 }

program_r:
   /* nothing */ { [] }
 | program_r fdecl { FuncDef($2)::$1 }
 | program_r vdecl { GlobalVar($2)::$1 }

/* List of all return types */
ptype_ret:
    INT 					{ RInt }
  | FLOAT       	{ RFloat }
  | BOOL 					{ RBoolean } 
  | CHAR 					{ RChar }
  | STRING 	      { RString }
  | TABLE       	{ RTable }
  | CELL 					{ RCell }
  | LINE 					{ RLine }
  | VOID {RVoid}

/* List of all types */
ptype_var:
    INT 					{ Int }
  | FLOAT       	{ Float }
  | BOOL 					{ Boolean } 
  | CHAR 					{ Char }
  | STRING 	      { String }
  | TABLE       	{ Table }
  | CELL 					{ Cell }
  | LINE 					{ Line }

/* table[range_op,range_op] range_op can be '@' or '1~2' */
range_op:
    WILDCARD 						{ (All,All) }
  | expr RANGE expr 		{ ($1,$3) }

/* e.g. func int some_func (int a, int b, float c) { Table foo; do_something; } */
fdecl:
   FUNC ptype_ret ID LPAREN formals_opt RPAREN block_stmt
     		{{ fname = FuncSig($2,$3);
				   formals = $5;
  		                    body= $7; 
			  }}

/* Parameters of a function */
formals_opt:
  /* nothing */ 	{ [] }
  | formal_list   { List.rev $1 }

formal_list:
    ptype_var ID    								{ [VarSig($1,$2)] }
  | formal_list COMMA ptype_var ID 	{ VarSig($3,$4) :: $1 }

/* List of Variables declared -- used in fdecl*/
/*vdecl_list:*/
  /* nothing    	 { [] }*/
 /* | vdecl_list vdecl { $2::$1 }*/


/* we allow variable initialization to default (null or false) or specific values */
/* Default initiliazations: Keep in mind that our language will default set all of */
/* these to a value of "NULL" except for bool, which has a default value of "false" */
vdecl: 
    ptype_var ID SEMI    							{ NullInit($1,$2) }                 /* default initialization*/
  | ptype_var ID ASSIGN expr SEMI     { VarInit($1,$2,$4) }               /* Explicit initialization value*/
  /* table default initialization*/
  | ptype_var ID ASSIGN LPAREN LSQPAREN expr COMMA expr RSQPAREN COMMA ptype_var RPAREN SEMI { TableInit($1,$2,$6,$8,$11) }

block_stmt:
    LBRACE stmt_list RBRACE { Block (List.rev $2) }

/* List of Statements declared -- used in fdecl*/
stmt_list:
  /* nothing */  		{ [] }
  | stmt_list stmt 	{ $2::$1 }
		
stmt:
    expr SEMI 							{ Expr($1) }
  | PRINT expr SEMI   		  { Print($2) }      /*print a table,a line a cell or any other literals*/
  | RETURN expr SEMI   	    { Return($2) }
  | BREAK SEMI   	    			{ Break }
  | IF LPAREN expr RPAREN block_stmt %prec NOELSE 				{ If($3,$5,NoElseMark) }
  | IF LPAREN expr RPAREN block_stmt ELSE block_stmt   		{ If($3,$5,$7) }
  | FOR LPAREN expr SEMI expr SEMI expr RPAREN block_stmt { For($3,$5,$7,$9) }
  | WHILE LPAREN expr RPAREN block_stmt 									{ While($3,$5) }
  | vdecl {VarDecl($1)}

literal:
    INTLITERAL          { IntL($1) }
  | FLOATLITERAL        { FloatL($1) }
  | CHARLITERAL         { CharL($1) }
  | STRINGLITERAL       { StringL($1) }
  | BOOLLITERAL         { BoolL($1) }  
     
                                         
expr:
    literal                     { Literal($1) }
  | ID                          { Id($1) }
  | ID LSQPAREN range_op COMMA range_op RSQPAREN  { TableAsTable($1,fst $3,snd $3,fst $5,snd $5) } /* table[1~2,3~4] */
  | ID LSQPAREN expr COMMA range_op RSQPAREN 	  { TableAsLine_R($1,$3,fst $5,snd $5) } /* table[3,1~2] */
  | ID LSQPAREN range_op COMMA expr RSQPAREN 	  { TableAsLine_L($1,fst $3,snd $3,$5) } /* table[2~3,5] */
  | ID LSQPAREN expr COMMA expr RSQPAREN 	  { TableAsCell($1,$3,$5) }  /* table[3,5] */
  | REF ID LSQPAREN expr COMMA expr RSQPAREN 	  { TableAsCellValue($2,$4,$6) }  /* ^table[3,5]*/
  | ID LSQPAREN range_op RSQPAREN  		  { LineAsLine($1,fst $3,snd $3) } /* line[1~2] */
  | ID LSQPAREN expr RSQPAREN  			  { LineAsCell($1,$3) } /* line[1] */
  | REF ID LSQPAREN expr RSQPAREN  		  { LineAsCellValue($2,$4) } /* ^line[1] */
  | REF ID           		                  { CellAsCell($2) } /* ^cell */
  | MINUS expr %prec TIMES   			  { UnaryMinus($2) }

  | expr PLUS   expr 		{ Binop($1, Add, $3) }
  | expr MINUS  expr 		{ Binop($1, Sub, $3) }
  | expr TIMES  expr 		{ Binop($1, Mult, $3) }
  | expr DIVIDE expr 		{ Binop($1, Div, $3) }
  | expr MOD 	expr			{ Binop($1, Mod, $3) }
  | expr EQ     expr 		{ Binop($1, Equal, $3) }
  | expr NEQ    expr 		{ Binop($1, Neq, $3) }
  | expr LT     expr 		{ Binop($1, Less_than, $3) }
  | expr LEQ    expr 		{ Binop($1, Leq, $3) }
  | expr GT     expr    { Binop($1, Greater_than, $3) }
  | expr GEQ    expr 		{ Binop($1, Geq, $3) }

  | expr AND    expr 		{ Binop($1, And, $3) }
  | expr OR     expr 		{ Binop($1, Or, $3) }
  | NOT expr            { Uniop (Not, $2)}
  | lvalue ASSIGN expr 	{ Assign($1,$3) } 
  | ID LPAREN actuals_opt RPAREN { Call($1,$3) }            /* Function Call */
  | LPAREN expr RPAREN 	{ $2 }  
  | NULL   							{ Null }
	
actuals_opt:
  /* nothing */  		{ [] }
  | actuals_list  	{ List.rev $1 }


actuals_list:
    expr                    { [$1] }
  | actuals_list COMMA expr { $3::$1 }


lvalue:
    ID                          		  { Id($1) }
  | REF ID           				  { NewTotalCell($2) }   

  | REF ID LSQPAREN range_op COMMA range_op RSQPAREN  { TableAsTable($2,fst $4,snd $4,fst $6,snd $6) } /* 
  | REF ID LSQPAREN expr COMMA range_op RSQPAREN  { TableAsLine_R($2,$4,fst $6,snd $6) } /* table[3,1~2] */
  | REF ID LSQPAREN range_op COMMA expr RSQPAREN  { TableAsLine_L($2,fst $4,snd $4,$6) } /* table[2~3,5] */
  | REF ID LSQPAREN expr COMMA expr RSQPAREN 	  { TableAsCellValue($2,$4,$6) }  /* ^table[3,5]*/

  | REF ID LSQPAREN expr RSQPAREN  		  { LineAsCellValue($2,$4) } /* ^line[1] */
  | REF ID LSQPAREN range_op RSQPAREN  		  { LineAsLine($2,fst $4,snd $4) } /* line[1~2] */


