#################################################################
# testall.sh													#
# This script will run all test files in "test_input"			#
# and compare their outputs to the expected outputs in			#
# "test_output."												#
#																#
# Any differences between input/expected output will be			#	
# saved in test_results.out										#
#################################################################


#################################################################
#					Variables and Config						#
#################################################################

# Based on your shell, may need to edit this path to Bourne shell
#!/usr/bin/sh		 

#dos2unix testall.sh					# For windows users

# Start with a clean output file
RESULTS=test_results.out
rm -f ${RESULTS}

# Different File Types
TAML=test*.taml
SCANNER=test*.scanner
PARSER=test*.parser
SAST=test*.sast
JAVA=test*.java
CLASS=test*.class
OUT=test*.out
 
# Our TaML Main Program
PROGRAM=./taml

NUM_TESTS=$(echo `ls test-output | wc -l`)
NUM_ERROR=0



#################################################################
#						Main Methods							#
#################################################################


### Runs all tests (.taml files) and saves outputs to each    ###
### stage with their appropriate extensions	in temp location  ###	
runTestFiles() {
	for FILE in test-input/${TAML}
	do
		SCANNER_TEMP_FILE=$(echo ${FILE} | sed 's/taml/scanner/')
		${PROGRAM} -s < ${FILE} > ${SCANNER_TEMP_FILE} || {
			echo "${FILE} failed to execute during scanning"
		}
		
		PARSER_TEMP_FILE=$(echo ${FILE} | sed 's/taml/parser/')
		${PROGRAM} -a < ${FILE} > ${PARSER_TEMP_FILE} || {
			echo "${FILE} failed to execute during parsing"
		}

 		SAST_TEMP_FILE=$(echo ${FILE} | sed 's/taml/sast/')
		${PROGRAM} -j < ${FILE} > ${SAST_TEMP_FILE} || {
			echo "${FILE} failed to execute during parsing"
		}

        JPP_TEMP_FILE=$(echo ${FILE} | sed 's/taml/java/')
        JPP_CLASS_NAME=$(echo ${FILE} | sed 's/\.taml// ' )
        JPP_CLASS_NAME=$(echo ${JPP_CLASS_NAME} | sed 's/test-input\/// ' )
        ${PROGRAM} -c ${JPP_CLASS_NAME} < ${FILE} > ${JPP_TEMP_FILE} || {
        	echo "${FILE} failed to execute during generating java file"
        }

	done
}

movejavafile(){
    for FILE in test-input/${JAVA}
    do
        cp $FILE java_lib/java_lib/
    done 

}

compileJavaFile(){  
	for FILE in java_lib/java_lib/${JAVA}
   	do
		JAVA_FILE=$(echo ${FILE} | sed 's/java_lib\/java_lib\//java_lib\//')
#  		echo ${JAVA_FILE}
   		cd java_lib
   		javac  ${JAVA_FILE}
   		cd ..
   done
}

runJavaClass() {
    for FILE in java_lib/java_lib/${CLASS}
    do
   		CLASS_FILE=$(echo ${FILE} | sed 's/java_lib\/java_lib\//java_lib\//')
   		OUT_FILE=$(echo ${CLASS_FILE} | sed 's/\.class/\.out/')
   		CLASS_FILE=$(echo ${CLASS_FILE} | sed 's/\.class//')
 # 		echo ${CLASS_FILE}
   		cd java_lib
   		java  ${CLASS_FILE} >> ${OUT_FILE}
   		cp ${OUT_FILE} ../test-input/
   		cd ..
   done
}

moveJavaResult(){
	for FILE in java_lib/java_lib/${JAVA}
	do
		cp $FILE java_out_result/
		rm $FILE
	done
}

moveOutResult(){
	for FILE in java_lib/java_lib/${OUT}
	do
		cp $FILE java_out_result/
		rm $FILE
	done
}


#################################################################
#	   				  Comparing Outputs							#
#################################################################

### Given an input file, compares it to the respective 		  ###
### output "golden" file, and outputs differences to RESULTS  ###
diffFiles() {
	IN_FILE=$1
	OUT_FILE=$(echo $1 | sed 's/input/output/')
 	diff -b -q ${IN_FILE} ${OUT_FILE} || {
 		echo "FAILED: Output Mismatch in ${IN_FILE}" >> ${RESULTS}
 		diff -b ${IN_FILE} ${OUT_FILE} >> ${RESULTS} 
 		echo "" >> ${RESULTS}
 		NUM_ERROR=$(( NUM_ERROR + 1 ))
 	}
}


### Test all Scanner output files and outputs results to log  ###
scannerResults(){
	echo "Comparing scanner files..."
	echo "Results of the Scanner test" >> $RESULTS
	for FILE in test-input/${SCANNER}
	do
		diffFiles ${FILE} 
	done
	echo "**************************************" >> ${RESULTS}
}


### Test all Parser output files and outputs results to log   ###
parserResults(){
	echo "Comparing parser files..."
	echo "Results of the Parser test" >> ${RESULTS}
	for FILE in test-input/${PARSER}
	do
		diffFiles ${FILE} 
	done
	echo "**************************************" >> ${RESULTS}
}


### Test all .java output files and outputs results to log   ###
sastResults(){
	echo "Comparing sast files..."
	echo "Results of the sast test" >> ${RESULTS}
	for FILE in test-input/${SAST}
	do
		diffFiles ${FILE} 
	done
	echo "**************************************" >> ${RESULTS}
}


javaResults(){
	echo "Comparing java files..."
	echo "Results of the java test" >> ${RESULTS}
	for FILE in test-input/${JAVA}
	do
		diffFiles ${FILE} 
	done
	echo "**************************************" >> ${RESULTS}
}


### Test all final output files (.out) and outputs results to log   ###
finalResults(){
	echo "Comparing out files..."
	echo "Results of the final output test" >> ${RESULTS}
	for FILE in test-input/${OUT}
	do
		diffFiles ${FILE} 
	done
	echo "**************************************" >> ${RESULTS}
}



#################################################################
#	   				  Run the Program							#
#################################################################

echo "See test_results.out for detailed results"

runTestFiles
scannerResults
parserResults
sastResults
javaResults

movejavafile
compileJavaFile
runJavaClass

finalResults

rm test-input/${SCANNER}
rm test-input/${PARSER}
rm test-input/${SAST}
rm test-input/${JAVA}
rm test-input/${OUT}

#moveJavaResult
#moveOutResult
rm java_lib/java_lib/${JAVA}
rm java_lib/java_lib/${CLASS}
rm java_lib/java_lib/${OUT}


echo "There were ${NUM_ERROR} out of ${NUM_TESTS} test cases which failed" 
echo "**************************************" >> ${RESULTS}
echo "There were ${NUM_ERROR} out of ${NUM_TESTS} test cases which failed" >> ${RESULTS}
